@@ -17,36 +17,33 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _CHEESE_FLASH_H_
-#define _CHEESE_FLASH_H_
+#ifndef CHEESE_FLASH_H_
+#define CHEESE_FLASH_H_
 
+#include <gtk/gtk.h>
 #include <glib-object.h>
 
 G_BEGIN_DECLS
 
+/**
+ * CheeseFlash:
+ *
+ * Use the accessor functions below.
+ */
+struct _CheeseFlash
+{
+  /*< private >*/
+  GtkWindow parent_instance;
+  void *unused;
+};
+
 #define CHEESE_TYPE_FLASH (cheese_flash_get_type ())
-#define CHEESE_FLASH(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), CHEESE_TYPE_FLASH, CheeseFlash))
-#define CHEESE_FLASH_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), CHEESE_TYPE_FLASH, CheeseFlashClass))
-#define CHEESE_IS_FLASH(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CHEESE_TYPE_FLASH))
-#define CHEESE_IS_FLASH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CHEESE_TYPE_FLASH))
-#define CHEESE_FLASH_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), CHEESE_TYPE_FLASH, CheeseFlashClass))
+G_DECLARE_FINAL_TYPE (CheeseFlash, cheese_flash, CHEESE, FLASH, GtkWindow)
 
-typedef struct
-{
-  GObjectClass parent_class;
-} CheeseFlashClass;
-
-typedef struct
-{
-  GObject parent_instance;
-} CheeseFlash;
-
-GType        cheese_flash_get_type (void) G_GNUC_CONST;
 CheeseFlash *cheese_flash_new (void);
-
 void cheese_flash_fire (CheeseFlash *flash,
                         GdkRectangle *rect);
 
 G_END_DECLS
 
-#endif /* _CHEESE_FLASH_H_ */
+#endif /* CHEESE_FLASH_H_ */
