@@ -24,7 +24,9 @@
 
 typedef enum {
   SCREENSHOT_RESPONSE_SAVE,
-  SCREENSHOT_RESPONSE_COPY
+  SCREENSHOT_RESPONSE_COPY,
+  SCREENSHOT_RESPONSE_BACK
+
 } ScreenshotResponse;
 
 typedef void (*SaveScreenshotCallback) (ScreenshotResponse response, gpointer *user_data);
@@ -38,6 +40,7 @@ typedef struct {
   GtkWidget *filename_entry;
   GtkWidget *save_button;
   GtkWidget *copy_button;
+  GtkWidget *back_button;
 
   gint drag_x;
   gint drag_y;
@@ -47,14 +50,14 @@ typedef struct {
 }  ScreenshotDialog;
 
 ScreenshotDialog *screenshot_dialog_new          (GdkPixbuf              *screenshot,
-						  char                   *initial_uri,
-						  SaveScreenshotCallback f,
-						  gpointer               user_data);
+                                                  char                   *initial_uri,
+                                                  SaveScreenshotCallback f,
+                                                  gpointer               user_data);
 
 char             *screenshot_dialog_get_uri      (ScreenshotDialog *dialog);
 char             *screenshot_dialog_get_folder   (ScreenshotDialog *dialog);
 char             *screenshot_dialog_get_filename (ScreenshotDialog *dialog);
 void              screenshot_dialog_set_busy     (ScreenshotDialog *dialog,
-						  gboolean          busy);
+                                                  gboolean          busy);
 
 #endif /* __SCREENSHOT_DIALOG_H__ */
