@@ -322,8 +322,6 @@ screenshot_fallback_fire_flash (GdkWindow *window,
 
   flash = cheese_flash_new ();
   cheese_flash_fire (flash, &rect);
-
-  g_object_unref (flash);
 }
 
 GdkWindow *
@@ -381,7 +379,7 @@ static GdkPixbuf *
 screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
 {
   GdkWindow *root, *wm_window = NULL;
-  GdkPixbuf *screenshot;
+  GdkPixbuf *screenshot = NULL;
   GdkRectangle real_coords, screenshot_coords;
   Window wm;
   GtkBorder frame_offset = { 0, 0, 0, 0 };
@@ -434,7 +432,6 @@ screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
   if (screenshot_config->include_border && (wm != None))
     {
       XRectangle *rectangles;
-      GdkPixbuf *tmp;
       int rectangle_count, rectangle_order, i;
 
       /* we must use XShape to avoid showing what's under the rounder corners
@@ -447,10 +444,11 @@ screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
                                         &rectangle_order);
       if (rectangles && rectangle_count > 0)
         {
+          int scale_factor = gdk_window_get_scale_factor (wm_window);
           gboolean has_alpha = gdk_pixbuf_get_has_alpha (screenshot);
-
-          tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-                                screenshot_coords.width, screenshot_coords.height);
+          GdkPixbuf *tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
+                                           gdk_pixbuf_get_width (screenshot),
+                                           gdk_pixbuf_get_height (screenshot));
           gdk_pixbuf_fill (tmp, 0);
 
           for (i = 0; i < rectangle_count; i++)
@@ -464,11 +462,15 @@ screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
                * areas for the invisible borders themselves.
                * In that case, trim every rectangle we get by the offset between the
                * WM window size and the frame extents.
+               *
+               * Note that the XShape values are in actual pixels, whereas the GDK
+               * ones are in display pixels (i.e. scaled), so we need to apply the
+               * scale factor to the former to use display pixels for all our math.
                */
-              rec_x = rectangles[i].x;
-              rec_y = rectangles[i].y;
-              rec_width = rectangles[i].width - (frame_offset.left + frame_offset.right);
-              rec_height = rectangles[i].height - (frame_offset.top + frame_offset.bottom);
+              rec_x = rectangles[i].x / scale_factor;
+              rec_y = rectangles[i].y / scale_factor;
+              rec_width = rectangles[i].width / scale_factor - (frame_offset.left + frame_offset.right);
+              rec_height = rectangles[i].height / scale_factor - (frame_offset.top + frame_offset.bottom);
 
               if (real_coords.x < 0)
                 {
@@ -490,19 +492,20 @@ screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
               if (screenshot_coords.y + rec_y + rec_height > gdk_screen_height ())
                 rec_height = gdk_screen_height () - screenshot_coords.y - rec_y;
 
-              for (y = rec_y; y < rec_y + rec_height; y++)
+              /* Undo the scale factor in order to copy the pixbuf data pixel-wise */
+              for (y = rec_y * scale_factor; y < (rec_y + rec_height) * scale_factor; y++)
                 {
                   guchar *src_pixels, *dest_pixels;
                   gint x;
 
                   src_pixels = gdk_pixbuf_get_pixels (screenshot)
                              + y * gdk_pixbuf_get_rowstride(screenshot)
-                             + rec_x * (has_alpha ? 4 : 3);
+                             + rec_x * scale_factor * (has_alpha ? 4 : 3);
                   dest_pixels = gdk_pixbuf_get_pixels (tmp)
                               + y * gdk_pixbuf_get_rowstride (tmp)
-                              + rec_x * 4;
+                              + rec_x * scale_factor * 4;
 
-                  for (x = 0; x < rec_width; x++)
+                  for (x = 0; x < rec_width * scale_factor; x++)
                     {
                       *dest_pixels++ = *src_pixels++;
                       *dest_pixels++ = *src_pixels++;
@@ -516,8 +519,7 @@ screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
                 }
             }
 
-          g_object_unref (screenshot);
-          screenshot = tmp;
+          g_set_object (&screenshot, tmp);
 
           XFree (rectangles);
         }
@@ -528,8 +530,8 @@ screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
    * screenshot */
   if (screenshot_config->include_pointer && !rectangle)
     {
-      GdkCursor *cursor;
-      GdkPixbuf *cursor_pixbuf;
+      g_autoptr(GdkCursor) cursor = NULL;
+      g_autoptr(GdkPixbuf) cursor_pixbuf = NULL;
 
       cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_LEFT_PTR);
       cursor_pixbuf = gdk_cursor_get_image (cursor);
@@ -575,9 +577,6 @@ screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
                                     GDK_INTERP_BILINEAR,
                                     255);
             }
-
-          g_object_unref (cursor_pixbuf);
-          g_object_unref (cursor);
         }
     }
 
@@ -586,14 +585,14 @@ screenshot_fallback_get_pixbuf (GdkRectangle *rectangle)
   return screenshot;
 }
 
-GdkPixbuf *
-screenshot_get_pixbuf (GdkRectangle *rectangle)
+static GdkPixbuf *
+screenshot_shell_get_pixbuf (GdkRectangle *rectangle)
 {
+  g_autoptr(GError) error = NULL;
+  g_autofree gchar *path = NULL, *filename = NULL, *tmpname = NULL;
   GdkPixbuf *screenshot = NULL;
-  gchar *path, *filename, *tmpname;
   const gchar *method_name;
   GVariant *method_params;
-  GError *error = NULL;
   GDBusConnection *connection;
 
   path = g_build_filename (g_get_user_cache_dir (), "gnome-screenshot", NULL);
@@ -650,18 +649,28 @@ screenshot_get_pixbuf (GdkRectangle *rectangle)
       g_unlink (filename);
     }
 
-  if (error != NULL)
+  return screenshot;
+}
+
+GdkPixbuf *
+screenshot_get_pixbuf (GdkRectangle *rectangle)
+{
+  GdkPixbuf *screenshot = NULL;
+  gboolean force_fallback;
+
+  force_fallback = g_getenv ("GNOME_SCREENSHOT_FORCE_FALLBACK") != NULL;
+  if (!force_fallback)
     {
-      g_message ("Unable to use GNOME Shell's builtin screenshot interface, "
-                 "resorting to fallback X11.");
-      g_error_free (error);
-
-      screenshot = screenshot_fallback_get_pixbuf (rectangle);
+      screenshot = screenshot_shell_get_pixbuf (rectangle);
+      if (!screenshot)
+        g_message ("Unable to use GNOME Shell's builtin screenshot interface, "
+                   "resorting to fallback X11.");
     }
+  else
+    g_message ("Using fallback X11 as requested");
 
-  g_free (path);
-  g_free (tmpname);
-  g_free (filename);
+  if (!screenshot)
+    screenshot = screenshot_fallback_get_pixbuf (rectangle);
 
   return screenshot;
 }
@@ -709,19 +718,16 @@ screenshot_show_dialog (GtkWindow   *parent,
 void
 screenshot_display_help (GtkWindow *parent)
 {
-  GError *error = NULL;
+  g_autoptr(GError) error = NULL;
 
   gtk_show_uri (gtk_window_get_screen (parent),
                 "help:gnome-help/screen-shot-record",
                 gtk_get_current_event_time (), &error);
 
   if (error)
-    {
-      screenshot_show_dialog (parent,
-                              GTK_MESSAGE_ERROR,
-                              GTK_BUTTONS_OK,
-                              _("Error loading the help page"),
-                              error->message);
-      g_error_free (error);
-    }
+    screenshot_show_dialog (parent,
+                            GTK_MESSAGE_ERROR,
+                            GTK_BUTTONS_OK,
+                            _("Error loading the help page"),
+                            error->message);
 }
